#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

install_requires = \
['django>=3']

extras_require = \
{'dev': ['black', 'isort', 'pre-commit'], 'test': ['tox']}

setup(name='django-lgi',
      version='0.1.2',
      description='Django Lambda Gateway Interface',
      author='Jarosław Wygoda',
      author_email='jaroslaw@wygoda.me',
      url='https://github.com/jwygoda/django-lgi',
      py_modules=['lgi'],
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
